unit UCFileGrep;

{ UCFileGrep is a component for simple text search in files.
  Please refer to UCFileGrep.txt for more information.
  Written for Delphi 5, but you may use it with D3 and D4 too.
  Freeware by UCSoft <info@ucsoft.de>
  Improved version 1.1 - 21.3.2001
  http://www.ucsoft.de
  }
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TUCFileGrep = class(TComponent)
  private
    { Private-Deklarationen }
    FCaseSensitive : boolean;
    FFilename : string;
    FOffset : integer;
    FSearchStr : string;
    FBlockSize : integer;
    procedure SetFilename(NewFilename : string);
    procedure SetSearchStr(NewSearchStr : string);
    procedure SetOffset(NewOffset : integer);
    procedure SetBlockSize(NewBlockSize : integer);
  protected
    { Protected-Deklarationen }
  public
    { Public-Deklarationen }
    function Search : integer;
    constructor Create(AOwner: TComponent); override;
  published
    { Published-Deklarationen }
    property BlockSize : integer read FBlockSize write SetBlockSize;
    property CaseSensitive : boolean read FCaseSensitive write FCaseSensitive;
    property Offset : integer read FOffset write SetOffset;
    property Filename : string read FFilename write SetFilename;
    property SearchStr : string read FSearchStr write SetSearchStr;
  end;

procedure Register;

implementation

{$R *.RES}

constructor TUCFileGrep.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FOffset:=0;
  FCaseSensitive:=false;
  FBlockSize:=16384;
end;

procedure TUCFileGrep.SetFilename(NewFilename : string);
begin
  if NewFilename<>FFilename then
  begin
    FFilename:=NewFilename;
    Offset:=0;
  end;
end;

procedure TUCFileGrep.SetSearchStr(NewSearchStr : string);
begin
  if NewSearchStr<>FSearchStr then
  begin
    FSearchStr:=NewSearchStr;
    Offset:=0;
    if FBlockSize<(Length(FSearchStr)*2) then
      FBlockSize:=Length(FSearchStr)*2;
  end;
end;

procedure TUCFileGrep.SetOffset(NewOffset : integer);
begin
  if (NewOffset<>FOffset) and (NewOffset>=0) then
    FOffset:=NewOffset;
end;

function TUCFileGrep.Search : integer;
  var
  f : file;
  Read : integer;
  Buffer : PChar;
  fp : integer;
  Found : boolean;
  ps,pf : PChar;
  Stop : boolean;
  PStr : PChar;
  st : PChar;
  StLen : integer;
  ch : Char;
  sr : integer;

begin
  Result:=-1;
  if not FileExists(FFilename) then
    Exit;

  stLen:=Length(FSearchStr);
  GetMem(st,stLen+1);
  StrPCopy(st,FSearchStr);
  if not FCaseSensitive then
    AnsiStrLower(st);

  // Alloc memory for buffer
  try
    GetMem(Buffer,FBlockSize);
  except
    on E : Exception do
    begin
      ShowMessage(E.Message);
      Exit;
    end;
  end;

  fp:=FOffset;
  // Open file
  AssignFile(f,FFilename);
  Reset(f,1);
  Seek(f,FOffset);

  Found:=false;
  Stop:=false;

  while (not Stop) and (not Found) do
  begin
    fp:=FilePos(f);
    BlockRead(f,Buffer^,FBlockSize,Read);
    if Read<>FBlockSize then
      Stop:=true; // Last block of file

    PStr:=Buffer;
    repeat
      pf:=StrScan(PStr,st^);     // Search first character
      if pf<>nil then
      begin
        ps:=pf+stLen;
        ch:=ps^;
        ps^:=#0;
        if FCaseSensitive then
          sr:=AnsiStrComp(pf,st)
        else
          sr:=AnsiStrIComp(pf,st);
        if sr=0 then
        begin
          Found:=true;
          fp:=fp+pf-Buffer;
        end;
        ps^:=ch;
        PStr:=ps;
      end
      else
        PStr:=PStr+1;
    until Found or (PStr>=Buffer+(Read-(stLen+2)));
    if not Found then
    begin
      // Go backward (if SearchStr is separeted in this and next block)
      Seek(f,FilePos(f)-(Length(FSearchStr)+2));
      fp:=FilePos(f);
    end;
  end;

  FreeMem(st);
  FreeMem(Buffer);
  CloseFile(f);
  if Found then
    Result:=fp;
end;

procedure TUCFileGrep.SetBlockSize(NewBlockSize : integer);
begin
  if (NewBlockSize<>FBlockSize) and (NewBlockSize>=1024) then
  begin
    FBlockSize:=NewBlockSize;
    if FBlockSize<(Length(FSearchStr)*2) then
      FBlockSize:=Length(FSearchStr)*2;
  end;
end;

procedure Register;
begin
  RegisterComponents('UCSoft', [TUCFileGrep]);
end;

end.
